#!/bin/bash

## Eenvoudig voorbeeldscript, dat een lijst genereert met de geïnstalleerde
## pakketten en programma's. Zie c’t 8-9/2022, p.132, www.ct.nl/softlink/2208132
## Dit kun je aan Cron toevoegen of door het back-upprogramma voor het starten
## van een back-up uitvoeren.

## Locatie waar lijs opgeslagen moet worden. Aanpassen naar wens.
## De ~ staat voor de hom-directory van de uitvoerende user. Bij de 
## user cttest is dat /home/cttest. Weet je het niet zeker, dan absolute
## paden opgeven. De directory moet al bestaan. 
## Voorbeelden
## LISTDIR=~
## LISTDIR=~/backups
## LISTDIR=~/backups/pakketlijsten

LISTDIR=~

## Debian, Ubuntu, Linux Mint en dergelijke,
## Lijst met apt generegen.
APT_LIST=yes

## Fedora, Red Hat en dergelijke (niet openSUSE)
## Lijst met dnf genereren.
DNF_LIST=yes

## Arch Linux, Manjaro en dergelijke.
## Lijst met pacman genereren.
PACMAN_LIST=yes

## Snap, met name bij Ubuntu
SNAP_LIST=yes

## Flatpak, als bijvoorbeeld Flathub gebruikt wordt.
FLATPAK_LIST=yes

## Lijstbestand bij iedere aanroep overschrijven?
## yes - bestand apt.list, flatpak.list, snap.list wordt overschreven
## no  - bestand met datum wordt aangemaakt, bijvoorbeeld apt_2022-06-24-09-44-30.list
## Default no 
OVERWRITE_FILE=no

## Wordt het script tweemaal in precies dezelfde seconde uitgevoerd, dan wordt
## het bestand ook bij "no" overschreven.

# Hulpvariabelen leeg initialiseren
NAME=""
BINARY=""

# Controle of de directory aanwezig en beschrijfbaar is
if [ -z ${LISTDIR} ]; then
	echo "Fout: geen directory aangegeven."
	exit 1
elif [ ! -d $LISTDIR ]; then
	echo "Fout: directory ${LISTDIR} bestaat niet."
	exit 1
elif [ ! -w $LISTDIR ]; then
	echo "Fout: geen schrijfrechten voor directory ${LISTDIR}."
	exit 1
else
	echo "Genereer lijst van geïnstalleerd pakketten en apps"
	
	if [ "$OVERWRITE_FILE" == "yes" ]; then
		echo "Overschrijf aanwezige bestanden."
		FILENAME_SUFFIX=".list"
	else
		DATE=`date +"%F-%H-%M-%S"`
		FILENAME_SUFFIX="_${DATE}.list"
	fi
## Lijst van de handmatig geïnstalleerde pakketten
	## Debian, Ubuntu, Linux Mint etc.
	# apt list --manual-installed

	## apt, pakketmanager voor Debian, Ubuntu etc.
	if [ "$APT_LIST" == "yes" ]; then
		NAME=APT
		BINARY=apt
		if [ -z `command -v $BINARY` ]; then
			echo "Fout: ${BINARY} niet gevonden. Sla ${NAME} over."
		else
			echo "Genereren nieuwe $NAME-lijst."
			FILENAME="${LISTDIR}/${BINARY}${FILENAME_SUFFIX}"
			
			apt list --manual-installed > $FILENAME
			
		fi
	fi	
	## Snap, app-store van Ubuntu-maker Canonical
	if [ "$SNAP_LIST" == "yes" ]; then
		NAME=SNAP
		BINARY=snap
		if [ -z `command -v $BINARY` ]; then
			echo "Fout: ${BINARY} niet gevonden. Sla ${NAME} over."
		else
			echo "Genereren nieuwe $NAME-lijst."
			FILENAME="${LISTDIR}/${BINARY}${FILENAME_SUFFIX}"
			
			snap list > $FILENAME
			
		fi
	fi
	## Flatpak, onafhankelijk Linux-pakketformaat
	if [ "$FLATPAK_LIST" == "yes" ]; then
		NAME=Flatpak
		BINARY=flatpak
		if [ -z `command -v $BINARY` ]; then
			echo "Fout: ${BINARY} niet gevonden. Sla ${NAME} over."
		else
			echo "Genereren nieuwe $NAME-lijst."
			FILENAME="${LISTDIR}/${BINARY}${FILENAME_SUFFIX}"
			
			flatpak list --app --columns=application,installation > $FILENAME
		fi
	fi
	
	## DNF, pakketmanager voor Fedora
	if [ "$DNF_LIST" == "yes" ]; then
		NAME=DNF
		BINARY=dnf
		if [ -z `command -v $BINARY` ]; then
			echo "Fout: ${BINARY} niet gevonden. Sla ${NAME} over."
		else
			echo "Genereren nieuwe $NAME-lijst."
			FILENAME="${LISTDIR}/${BINARY}${FILENAME_SUFFIX}"
			
			dnf history userinstalled > $FILENAME
		fi
	fi
	
	## Pacman, pakketmanager voor Arch Linux
	if [ "$PACMAN_LIST" == "yes" ]; then
		NAME=Pacman
		BINARY=pacman
		if [ -z `command -v $BINARY` ]; then
			echo "Fout: ${BINARY} niet gevonden. Sla ${NAME} over."
		else
			echo "Genereren nieuwe $NAME-lijst."
			FILENAME="${LISTDIR}/${BINARY}${FILENAME_SUFFIX}"
			
			pacman -Qet > $FILENAME
		fi
	fi

fi 
